#!/usr/bin/env python3
#
# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)

import pickle

import kaldifeat


def test_default():
    opts = kaldifeat.FrameExtractionOptions()
    print(opts)
    assert opts.samp_freq == 16000
    assert opts.frame_shift_ms == 10.0
    assert opts.frame_length_ms == 25.0
    assert opts.dither == 1.0
    assert abs(opts.preemph_coeff - 0.97) < 1e-6
    assert opts.remove_dc_offset is True
    assert opts.window_type == "povey"
    assert opts.round_to_power_of_two is True
    assert abs(opts.blackman_coeff - 0.42) < 1e-6
    assert opts.snip_edges is True


def test_set_get():
    opts = kaldifeat.FrameExtractionOptions(samp_freq=22150)
    assert opts.samp_freq == 22150

    opts.samp_freq = 44100
    assert opts.samp_freq == 44100

    opts.frame_shift_ms = 20.5
    assert opts.frame_shift_ms == 20.5

    opts.frame_length_ms = 1
    assert opts.frame_length_ms == 1

    opts.dither = 0.5
    assert opts.dither == 0.5

    opts.preemph_coeff = 0.25
    assert opts.preemph_coeff == 0.25

    opts.remove_dc_offset = False
    assert opts.remove_dc_offset is False

    opts.window_type = "hanning"
    assert opts.window_type == "hanning"

    opts.round_to_power_of_two = False
    assert opts.round_to_power_of_two is False

    opts.blackman_coeff = 0.25
    assert opts.blackman_coeff == 0.25

    opts.snip_edges = False
    assert opts.snip_edges is False


def test_from_empty_dict():
    opts = kaldifeat.FrameExtractionOptions.from_dict({})
    opts2 = kaldifeat.FrameExtractionOptions()

    assert str(opts) == str(opts2)


def test_from_dict_partial():
    d = {"samp_freq": 10, "frame_shift_ms": 2}

    opts = kaldifeat.FrameExtractionOptions.from_dict(d)

    opts2 = kaldifeat.FrameExtractionOptions()
    assert str(opts) != str(opts2)

    opts2.samp_freq = 10
    assert str(opts) != str(opts2)

    opts2.frame_shift_ms = 2
    assert str(opts) == str(opts2)

    opts2.frame_shift_ms = 3
    assert str(opts) != str(opts2)


def test_from_dict_full_and_as_dict():
    opts = kaldifeat.FrameExtractionOptions()
    opts.samp_freq = 20
    opts.frame_length_ms = 100

    d = opts.as_dict()
    for key, value in d.items():
        assert value == getattr(opts, key)

    opts2 = kaldifeat.FrameExtractionOptions.from_dict(d)
    assert str(opts2) == str(opts)

    d["window_type"] = "hanning"
    opts3 = kaldifeat.FrameExtractionOptions.from_dict(d)
    assert opts3.window_type == "hanning"


def test_pickle():
    opts = kaldifeat.FrameExtractionOptions()
    opts.samp_freq = 44100
    opts.dither = 5.5
    data = pickle.dumps(opts)

    opts2 = pickle.loads(data)
    assert str(opts) == str(opts2)


def main():
    test_default()
    test_set_get()
    test_from_empty_dict()
    test_from_dict_partial()
    test_from_dict_full_and_as_dict()
    test_pickle()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)


import pickle

import torch

import kaldifeat


def test_default():
    opts = kaldifeat.SpectrogramOptions()
    print(opts)

    assert opts.frame_opts.samp_freq == 16000
    assert opts.frame_opts.frame_shift_ms == 10.0
    assert opts.frame_opts.frame_length_ms == 25.0
    assert opts.frame_opts.dither == 1.0
    assert abs(opts.frame_opts.preemph_coeff - 0.97) < 1e-6
    assert opts.frame_opts.remove_dc_offset is True
    assert opts.frame_opts.window_type == "povey"
    assert opts.frame_opts.round_to_power_of_two is True
    assert abs(opts.frame_opts.blackman_coeff - 0.42) < 1e-6
    assert opts.frame_opts.snip_edges is True

    assert opts.energy_floor == 0
    assert opts.raw_energy is True
    assert opts.device.type == "cpu"


def test_set_get():
    opts = kaldifeat.SpectrogramOptions(energy_floor=10)
    assert opts.energy_floor == 10

    opts.energy_floor = 1
    assert opts.energy_floor == 1

    opts.raw_energy = False
    assert opts.raw_energy is False

    opts.device = torch.device("cuda", 1)
    assert opts.device.type == "cuda"
    assert opts.device.index == 1


def test_set_get_frame_opts():
    opts = kaldifeat.SpectrogramOptions()

    opts.frame_opts.samp_freq = 44100
    assert opts.frame_opts.samp_freq == 44100

    opts.frame_opts.frame_shift_ms = 20.5
    assert opts.frame_opts.frame_shift_ms == 20.5

    opts.frame_opts.frame_length_ms = 1
    assert opts.frame_opts.frame_length_ms == 1

    opts.frame_opts.dither = 0.5
    assert opts.frame_opts.dither == 0.5

    opts.frame_opts.preemph_coeff = 0.25
    assert opts.frame_opts.preemph_coeff == 0.25

    opts.frame_opts.remove_dc_offset = False
    assert opts.frame_opts.remove_dc_offset is False

    opts.frame_opts.window_type = "hanning"
    assert opts.frame_opts.window_type == "hanning"

    opts.frame_opts.round_to_power_of_two = False
    assert opts.frame_opts.round_to_power_of_two is False

    opts.frame_opts.blackman_coeff = 0.25
    assert opts.frame_opts.blackman_coeff == 0.25

    opts.frame_opts.snip_edges = False
    assert opts.frame_opts.snip_edges is False


def test_from_empty_dict():
    opts = kaldifeat.SpectrogramOptions.from_dict({})
    opts2 = kaldifeat.SpectrogramOptions()

    assert str(opts) == str(opts2)


def test_from_dict_partial():
    d = {
        "energy_floor": 10.5,
        "frame_opts": {"window_type": "hanning"},
        "device": "cuda:2",
    }
    opts = kaldifeat.SpectrogramOptions.from_dict(d)
    assert opts.energy_floor == 10.5
    assert opts.device == torch.device("cuda", 2)
    assert opts.frame_opts.window_type == "hanning"

    frame_opts = kaldifeat.FrameExtractionOptions.from_dict(d["frame_opts"])
    assert str(opts.frame_opts) == str(frame_opts)


def test_from_dict_full_and_as_dict():
    opts = kaldifeat.SpectrogramOptions()
    opts.frame_opts.samp_freq = 12
    opts.device = "cuda:3"

    d = opts.as_dict()
    assert d["frame_opts"]["samp_freq"] == 12
    assert d["device"] == torch.device("cuda:3")

    frame_opts = kaldifeat.FrameExtractionOptions()
    frame_opts.samp_freq = 12
    assert d["frame_opts"] == frame_opts.as_dict()

    opts2 = kaldifeat.SpectrogramOptions.from_dict(d)
    assert str(opts2) == str(opts)

    d["device"] = torch.device("cuda", 10)
    opts3 = kaldifeat.SpectrogramOptions.from_dict(d)
    assert opts3.device == torch.device("cuda", 10)

    opts.device = "cuda:10"
    assert str(opts3) == str(opts)


def test_pickle():
    opts = kaldifeat.SpectrogramOptions()
    opts.energy_floor = 1
    opts.raw_energy = False
    opts.device = torch.device("cuda", 1)

    opts.frame_opts.samp_freq = 44100
    opts.frame_opts.snip_edges = False

    data = pickle.dumps(opts)

    opts2 = pickle.loads(data)
    assert str(opts) == str(opts2)


def test_device():
    opts = kaldifeat.SpectrogramOptions(device="cpu")
    assert opts.device == torch.device("cpu")

    opts = kaldifeat.SpectrogramOptions(device="cuda")
    assert opts.device == torch.device("cuda")

    opts = kaldifeat.SpectrogramOptions(device="cuda:1")
    assert opts.device == torch.device("cuda:1")
    print(opts)

    opts = kaldifeat.SpectrogramOptions(device=torch.device("cpu"))
    assert opts.device == torch.device("cpu")

    opts = kaldifeat.SpectrogramOptions(device=torch.device("cuda"))
    assert opts.device == torch.device("cuda")

    opts = kaldifeat.SpectrogramOptions(device=torch.device("cuda:3"))
    assert opts.device == torch.device("cuda:3")

    opts = kaldifeat.SpectrogramOptions(device=torch.device("cuda", 2))
    assert opts.device == torch.device("cuda", 2)


def main():
    test_default()
    test_set_get()
    test_set_get_frame_opts()
    test_from_empty_dict()
    test_from_dict_partial()
    test_from_dict_full_and_as_dict()
    test_pickle()
    test_device()


if __name__ == "__main__":
    main()



# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)


import _kaldifeat

from .offline_feature import OfflineFeature
from .online_feature import OnlineFeature


class Mfcc(OfflineFeature):
    def __init__(self, opts: _kaldifeat.MfccOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.Mfcc(opts)


class OnlineMfcc(OnlineFeature):
    def __init__(self, opts: _kaldifeat.MfccOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.OnlineMfcc(opts)

    def __setstate__(self, state):
        self.opts = _kaldifeat.MfccOptions.from_dict(state)
        self.computer = _kaldifeat.OnlineMfcc(self.opts)



#!/usr/bin/env python3

# Copyright      2021-2022  Xiaomi Corporation (authors: Fangjun Kuang)

import pickle
from pathlib import Path

import torch
from utils import get_devices, read_ark_txt, read_wave

import kaldifeat

cur_dir = Path(__file__).resolve().parent


def test_online_fbank(
    opts: kaldifeat.FbankOptions,
    wave: torch.Tensor,
    cpu_features: torch.Tensor,
):
    """
    Args:
      opts:
        The options to create the online fbank extractor.
      wave:
        The input 1-D waveform.
      cpu_features:
        The groud truth features that are computed offline
    """
    online_fbank = kaldifeat.OnlineFbank(opts)

    num_processed_frames = 0
    i = 0  # current sample index to feed
    while not online_fbank.is_last_frame(num_processed_frames - 1):
        while num_processed_frames < online_fbank.num_frames_ready:
            # There are new frames to be processed
            frame = online_fbank.get_frame(num_processed_frames)
            assert torch.allclose(
                frame.squeeze(0), cpu_features[num_processed_frames]
            )
            num_processed_frames += 1

        # Simulate streaming . Send a random number of audio samples
        # to the extractor
        num_samples = torch.randint(300, 1000, (1,)).item()

        samples = wave[i : (i + num_samples)]  # noqa
        i += num_samples
        if len(samples) == 0:
            online_fbank.input_finished()
            continue

        online_fbank.accept_waveform(16000, samples)

    assert num_processed_frames == online_fbank.num_frames_ready
    assert num_processed_frames == cpu_features.size(0)


def test_fbank_default():
    print("=====test_fbank_default=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        fbank = kaldifeat.Fbank(opts)

        features = fbank(wave)
        assert features.device.type == "cpu"
        assert torch.allclose(features, gt, rtol=1e-1)
        if cpu_features is None:
            cpu_features = features

        features = fbank(wave.to(device))
        assert features.device == device
        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    # Now for online fbank
    opts = kaldifeat.FbankOptions()
    opts.frame_opts.dither = 0
    opts.frame_opts.max_feature_vectors = 100

    test_online_fbank(opts, wave, cpu_features)


def test_fbank_htk():
    print("=====test_fbank_htk=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-htk.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.use_energy = True
        opts.htk_compat = True

        fbank = kaldifeat.Fbank(opts)

        features = fbank(wave)
        assert features.device.type == "cpu"
        assert torch.allclose(features, gt, rtol=1e-1)
        if cpu_features is None:
            cpu_features = features

        features = fbank(wave.to(device))
        assert features.device == device
        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    opts = kaldifeat.FbankOptions()
    opts.frame_opts.dither = 0
    opts.use_energy = True
    opts.htk_compat = True

    test_online_fbank(opts, wave, cpu_features)


def test_fbank_with_energy():
    print("=====test_fbank_with_energy=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-with-energy.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.use_energy = True

        fbank = kaldifeat.Fbank(opts)

        features = fbank(wave)
        assert torch.allclose(features, gt, rtol=1e-1)
        assert features.device.type == "cpu"
        if cpu_features is None:
            cpu_features = features

        features = fbank(wave.to(device))
        assert features.device == device
        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    opts = kaldifeat.FbankOptions()
    opts.frame_opts.dither = 0
    opts.use_energy = True

    test_online_fbank(opts, wave, cpu_features)


def test_fbank_40_bins():
    print("=====test_fbank_40_bins=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-40.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.mel_opts.num_bins = 40

        fbank = kaldifeat.Fbank(opts)

        features = fbank(wave)
        assert features.device.type == "cpu"
        assert torch.allclose(features, gt, rtol=1e-1)
        if cpu_features is None:
            cpu_features = features

        features = fbank(wave.to(device))
        assert features.device == device
        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    opts = kaldifeat.FbankOptions()
    opts.frame_opts.dither = 0
    opts.mel_opts.num_bins = 40

    test_online_fbank(opts, wave, cpu_features)


def test_fbank_40_bins_no_snip_edges():
    print("=====test_fbank_40_bins_no_snip_edges=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-40-no-snip-edges.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.mel_opts.num_bins = 40
        opts.frame_opts.snip_edges = False

        fbank = kaldifeat.Fbank(opts)

        features = fbank(wave)
        assert features.device.type == "cpu"
        assert torch.allclose(features, gt, rtol=1e-1)
        if cpu_features is None:
            cpu_features = features

        features = fbank(wave.to(device))
        assert features.device == device
        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    opts = kaldifeat.FbankOptions()
    opts.frame_opts.dither = 0
    opts.mel_opts.num_bins = 40
    opts.frame_opts.snip_edges = False

    test_online_fbank(opts, wave, cpu_features)


def test_fbank_chunk():
    print("=====test_fbank_chunk=====")

    filename = cur_dir / "test_data/test-1hour.wav"
    if filename.is_file() is False:
        print(
            f"Please execute {cur_dir}/test_data/run.sh "
            f"to generate {filename} before running tis test"
        )
        return

    for device in get_devices():
        print("device", device)
        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.mel_opts.num_bins = 40
        opts.frame_opts.snip_edges = False

        fbank = kaldifeat.Fbank(opts)
        wave = read_wave(filename)

        # You can use
        #
        #  $ watch -n 0.2 free -m
        #
        # to view memory consumption
        #
        # 100 frames per chunk
        features1 = fbank(wave, chunk_size=100 * 10)
        features2 = fbank(wave)
        assert torch.allclose(features1, features2)
        assert features1.device == features2.device
        assert features1.device.type == "cpu"

        if device.type == "cuda":
            wave = wave.to(device)
            features1 = fbank(wave, chunk_size=100 * 10)
            features2 = fbank(wave)
            assert torch.allclose(features1, features2)
            assert features1.device == features2.device
            assert features1.device == device


def test_fbank_batch():
    print("=====test_fbank_batch=====")
    for device in get_devices():
        print("device", device)
        wave0 = read_wave(cur_dir / "test_data/test.wav")
        wave1 = read_wave(cur_dir / "test_data/test2.wav")

        opts = kaldifeat.FbankOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        fbank = kaldifeat.Fbank(opts)

        features = fbank([wave0, wave1], chunk_size=10)

        features0 = fbank(wave0)
        features1 = fbank(wave1)

        assert torch.allclose(features[0], features0)
        assert torch.allclose(features[1], features1)

        if device.type == "cuda":
            wave0 = wave0.to(device)
            wave1 = wave1.to(device)

            features = fbank([wave0, wave1], chunk_size=10)

            features0 = fbank(wave0)
            features1 = fbank(wave1)

            assert torch.allclose(features[0], features0)
            assert torch.allclose(features[1], features1)


def test_pickle():
    for device in get_devices():
        opts = kaldifeat.FbankOptions()
        opts.use_energy = True
        opts.use_power = False
        opts.device = device

        fbank = kaldifeat.Fbank(opts)
        data = pickle.dumps(fbank)
        fbank2 = pickle.loads(data)

        assert str(fbank.opts) == str(fbank2.opts)

    opts = kaldifeat.FbankOptions()
    opts.use_energy = True
    opts.use_power = False

    fbank = kaldifeat.OnlineFbank(opts)
    data = pickle.dumps(fbank)
    fbank2 = pickle.loads(data)

    assert str(fbank.opts) == str(fbank2.opts)


if __name__ == "__main__":
    test_fbank_default()
    test_fbank_htk()
    test_fbank_with_energy()
    test_fbank_40_bins()
    test_fbank_40_bins_no_snip_edges()
    test_fbank_chunk()
    test_fbank_batch()
    test_pickle()



#!/usr/bin/env python3
#
# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)


import pickle

import torch

import kaldifeat


def test_default():
    opts = kaldifeat.FbankOptions()
    print(opts)
    assert opts.frame_opts.samp_freq == 16000
    assert opts.frame_opts.frame_shift_ms == 10.0
    assert opts.frame_opts.frame_length_ms == 25.0
    assert opts.frame_opts.dither == 1.0
    assert abs(opts.frame_opts.preemph_coeff - 0.97) < 1e-6
    assert opts.frame_opts.remove_dc_offset is True
    assert opts.frame_opts.window_type == "povey"
    assert opts.frame_opts.round_to_power_of_two is True
    assert abs(opts.frame_opts.blackman_coeff - 0.42) < 1e-6
    assert opts.frame_opts.snip_edges is True

    assert opts.mel_opts.num_bins == 23
    assert opts.mel_opts.low_freq == 20
    assert opts.mel_opts.high_freq == 0
    assert opts.mel_opts.vtln_low == 100
    assert opts.mel_opts.vtln_high == -500
    assert opts.mel_opts.debug_mel is False
    assert opts.mel_opts.htk_mode is False

    assert opts.use_energy is False
    assert opts.energy_floor == 0.0
    assert opts.raw_energy is True
    assert opts.htk_compat is False
    assert opts.use_log_fbank is True
    assert opts.use_power is True
    assert opts.device.type == "cpu"


def test_set_get():
    opts = kaldifeat.FbankOptions()
    opts.use_energy = True
    assert opts.use_energy is True

    opts.energy_floor = 1
    assert opts.energy_floor == 1

    opts.raw_energy = False
    assert opts.raw_energy is False

    opts.htk_compat = True
    assert opts.htk_compat is True

    opts.use_log_fbank = False
    assert opts.use_log_fbank is False

    opts.use_power = False
    assert opts.use_power is False

    opts.device = torch.device("cuda", 1)
    assert opts.device.type == "cuda"
    assert opts.device.index == 1


def test_set_get_frame_opts():
    opts = kaldifeat.FbankOptions()

    opts.frame_opts.samp_freq = 44100
    assert opts.frame_opts.samp_freq == 44100

    opts.frame_opts.frame_shift_ms = 20.5
    assert opts.frame_opts.frame_shift_ms == 20.5

    opts.frame_opts.frame_length_ms = 1
    assert opts.frame_opts.frame_length_ms == 1

    opts.frame_opts.dither = 0.5
    assert opts.frame_opts.dither == 0.5

    opts.frame_opts.preemph_coeff = 0.25
    assert opts.frame_opts.preemph_coeff == 0.25

    opts.frame_opts.remove_dc_offset = False
    assert opts.frame_opts.remove_dc_offset is False

    opts.frame_opts.window_type = "hanning"
    assert opts.frame_opts.window_type == "hanning"

    opts.frame_opts.round_to_power_of_two = False
    assert opts.frame_opts.round_to_power_of_two is False

    opts.frame_opts.blackman_coeff = 0.25
    assert opts.frame_opts.blackman_coeff == 0.25

    opts.frame_opts.snip_edges = False
    assert opts.frame_opts.snip_edges is False


def test_set_get_mel_opts():
    opts = kaldifeat.FbankOptions()

    opts.mel_opts.num_bins = 100
    assert opts.mel_opts.num_bins == 100

    opts.mel_opts.low_freq = 22
    assert opts.mel_opts.low_freq == 22

    opts.mel_opts.high_freq = 1
    assert opts.mel_opts.high_freq == 1

    opts.mel_opts.vtln_low = 101
    assert opts.mel_opts.vtln_low == 101

    opts.mel_opts.vtln_high = -100
    assert opts.mel_opts.vtln_high == -100

    opts.mel_opts.debug_mel = True
    assert opts.mel_opts.debug_mel is True

    opts.mel_opts.htk_mode = True
    assert opts.mel_opts.htk_mode is True


def test_from_empty_dict():
    opts = kaldifeat.FbankOptions.from_dict({})
    opts2 = kaldifeat.FbankOptions()

    assert str(opts) == str(opts2)


def test_from_dict_partial():
    d = {
        "energy_floor": 10.5,
        "htk_compat": True,
        "mel_opts": {"num_bins": 80, "vtln_low": 1},
        "frame_opts": {"window_type": "hanning"},
    }
    opts = kaldifeat.FbankOptions.from_dict(d)
    assert opts.energy_floor == 10.5
    assert opts.htk_compat is True
    assert opts.mel_opts.num_bins == 80
    assert opts.mel_opts.vtln_low == 1
    assert opts.frame_opts.window_type == "hanning"

    mel_opts = kaldifeat.MelBanksOptions.from_dict(d["mel_opts"])
    assert str(opts.mel_opts) == str(mel_opts)


def test_from_dict_full_and_as_dict():
    opts = kaldifeat.FbankOptions()
    opts.htk_compat = True
    opts.mel_opts.num_bins = 80
    opts.frame_opts.samp_freq = 10

    d = opts.as_dict()
    assert d["htk_compat"] is True
    assert d["mel_opts"]["num_bins"] == 80
    assert d["frame_opts"]["samp_freq"] == 10

    mel_opts = kaldifeat.MelBanksOptions()
    mel_opts.num_bins = 80
    assert d["mel_opts"] == mel_opts.as_dict()

    frame_opts = kaldifeat.FrameExtractionOptions()
    frame_opts.samp_freq = 10
    assert d["frame_opts"] == frame_opts.as_dict()

    opts2 = kaldifeat.FbankOptions.from_dict(d)
    assert str(opts2) == str(opts)

    d["htk_compat"] = False
    d["device"] = torch.device("cuda", 2)
    opts3 = kaldifeat.FbankOptions.from_dict(d)
    assert opts3.htk_compat is False
    assert opts3.device == torch.device("cuda", 2)


def test_pickle():
    opts = kaldifeat.FbankOptions()
    opts.use_energy = True
    opts.use_power = False
    opts.device = torch.device("cuda", 1)

    opts.frame_opts.samp_freq = 44100
    opts.mel_opts.num_bins = 100

    data = pickle.dumps(opts)

    opts2 = pickle.loads(data)
    assert str(opts) == str(opts2)


def main():
    test_default()
    test_set_get()
    test_set_get_frame_opts()
    test_set_get_mel_opts()
    test_from_empty_dict()
    test_from_dict_partial()
    test_from_dict_full_and_as_dict()
    test_pickle()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)

import pickle

import kaldifeat


def test_default():
    opts = kaldifeat.MelBanksOptions()
    print(opts)
    assert opts.num_bins == 25
    assert opts.low_freq == 20
    assert opts.high_freq == 0
    assert opts.vtln_low == 100
    assert opts.vtln_high == -500
    assert opts.debug_mel is False
    assert opts.htk_mode is False


def test_set_get():
    opts = kaldifeat.MelBanksOptions(num_bins=100)
    assert opts.num_bins == 100

    opts.num_bins = 200
    assert opts.num_bins == 200

    opts.low_freq = 22
    assert opts.low_freq == 22

    opts.high_freq = 1
    assert opts.high_freq == 1

    opts.vtln_low = 101
    assert opts.vtln_low == 101

    opts.vtln_high = -100
    assert opts.vtln_high == -100

    opts.debug_mel = True
    assert opts.debug_mel is True

    opts.htk_mode = True
    assert opts.htk_mode is True


def test_from_empty_dict():
    opts = kaldifeat.MelBanksOptions.from_dict({})
    opts2 = kaldifeat.MelBanksOptions()

    assert str(opts) == str(opts2)


def test_from_dict_partial():
    d = {"num_bins": 10, "debug_mel": True}

    opts = kaldifeat.MelBanksOptions.from_dict(d)

    opts2 = kaldifeat.MelBanksOptions()
    assert str(opts) != str(opts2)

    opts2.num_bins = 10
    assert str(opts) != str(opts2)

    opts2.debug_mel = True
    assert str(opts) == str(opts2)

    opts2.debug_mel = False
    assert str(opts) != str(opts2)


def test_from_dict_full_and_as_dict():
    opts = kaldifeat.MelBanksOptions()
    opts.num_bins = 80
    opts.vtln_high = 2

    d = opts.as_dict()
    for key, value in d.items():
        assert value == getattr(opts, key)

    opts2 = kaldifeat.MelBanksOptions.from_dict(d)
    assert str(opts2) == str(opts)

    d["htk_mode"] = True
    opts3 = kaldifeat.MelBanksOptions.from_dict(d)
    assert opts3.htk_mode is True


def test_pickle():
    opts = kaldifeat.MelBanksOptions()
    opts.num_bins = 100
    opts.low_freq = 22
    data = pickle.dumps(opts)

    opts2 = pickle.loads(data)
    assert str(opts) == str(opts2)


def main():
    test_default()
    test_set_get()
    test_from_empty_dict()
    test_from_dict_partial()
    test_from_dict_full_and_as_dict()
    test_pickle()


if __name__ == "__main__":
    main()



# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)


import _kaldifeat

from .offline_feature import OfflineFeature
from .online_feature import OnlineFeature


class Plp(OfflineFeature):
    def __init__(self, opts: _kaldifeat.PlpOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.Plp(opts)


class OnlinePlp(OnlineFeature):
    def __init__(self, opts: _kaldifeat.PlpOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.OnlinePlp(opts)

    def __setstate__(self, state):
        self.opts = _kaldifeat.PlpOptions.from_dict(state)
        self.computer = _kaldifeat.OnlinePlp(self.opts)



# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)


import _kaldifeat

from .offline_feature import OfflineFeature
from .online_feature import OnlineFeature


class Fbank(OfflineFeature):
    def __init__(self, opts: _kaldifeat.FbankOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.Fbank(opts)


class OnlineFbank(OnlineFeature):
    def __init__(self, opts: _kaldifeat.FbankOptions):
        super().__init__(opts)
        self.computer = _kaldifeat.OnlineFbank(opts)

    def __setstate__(self, state):
        self.opts = _kaldifeat.FbankOptions.from_dict(state)
        self.computer = _kaldifeat.OnlineFbank(self.opts)



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)

import pickle
from pathlib import Path

from utils import get_devices, read_ark_txt, read_wave

import kaldifeat

cur_dir = Path(__file__).resolve().parent


def test_spectrogram_default():
    print("=====test_spectrogram_default=====")
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.SpectrogramOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        spectrogram = kaldifeat.Spectrogram(opts)
        filename = cur_dir / "test_data/test.wav"
        wave = read_wave(filename).to(opts.device)

        features = spectrogram(wave)
        gt = read_ark_txt(cur_dir / "test_data/test-spectrogram.txt")

        #  assert torch.allclose(features.cpu(), gt, atol=1.1)
        print(features[1, 145:148], gt[1, 145:148])  # they are different


def test_spectrogram_no_snip_edges():
    print("=====test_spectrogram_no_snip_edges=====")
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.SpectrogramOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.frame_opts.snip_edges = False
        spectrogram = kaldifeat.Spectrogram(opts)
        filename = cur_dir / "test_data/test.wav"
        wave = read_wave(filename).to(opts.device)

        features = spectrogram(wave)
        gt = read_ark_txt(
            cur_dir / "test_data/test-spectrogram-no-snip-edges.txt"
        )

        #  assert torch.allclose(features.cpu(), gt, atol=1.5)
        print(features[1, 145:148], gt[1, 145:148])  # they are different


def test_pickle():
    for device in get_devices():
        opts = kaldifeat.SpectrogramOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.frame_opts.snip_edges = False

        spec = kaldifeat.Spectrogram(opts)
        data = pickle.dumps(spec)
        spec2 = pickle.loads(data)

        assert str(spec.opts) == str(spec2.opts)


if __name__ == "__main__":
    test_spectrogram_default()
    test_spectrogram_no_snip_edges()
    test_pickle()



import torch

from .torch_version import kaldifeat_torch_version

if torch.__version__.split("+")[0] != kaldifeat_torch_version.split("+")[0]:
    raise ImportError(
        f"kaldifeat was built using PyTorch {kaldifeat_torch_version}\n"
        f"But you are using PyTorch {torch.__version__} to run it"
    )

from pathlib import Path as _Path

from _kaldifeat import (
    FbankOptions,
    FrameExtractionOptions,
    MelBanksOptions,
    MfccOptions,
    PlpOptions,
    SpectrogramOptions,
    num_frames,
)

from .fbank import Fbank, OnlineFbank
from .mfcc import Mfcc, OnlineMfcc
from .offline_feature import OfflineFeature
from .online_feature import OnlineFeature
from .plp import OnlinePlp, Plp
from .spectrogram import Spectrogram

cmake_prefix_path = _Path(__file__).parent / "share" / "cmake"
del _Path
__version__ = '1.24'



# Copyright (c)  2021  Xiaomi Corporation (author: Fangjun Kuang)

import glob
import os
import platform
import shutil
import sys
from pathlib import Path

import setuptools
from setuptools.command.build_ext import build_ext


def is_for_pypi():
    ans = os.environ.get("KALDIFEAT_IS_FOR_PYPI", None)
    return ans is not None


def is_macos():
    return platform.system() == "Darwin"


def is_windows():
    return platform.system() == "Windows"


try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel

    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            # In this case, the generated wheel has a name in the form
            # kaldifeat-xxx-pyxx-none-any.whl
            if is_for_pypi() and not is_macos():
                self.root_is_pure = True
            else:
                # The generated wheel has a name ending with
                # -linux_x86_64.whl
                self.root_is_pure = False


except ImportError:
    bdist_wheel = None


def cmake_extension(name, *args, **kwargs) -> setuptools.Extension:
    kwargs["language"] = "c++"
    sources = []
    return setuptools.Extension(name, sources, *args, **kwargs)


class BuildExtension(build_ext):
    def build_extension(self, ext: setuptools.extension.Extension):
        # build/temp.linux-x86_64-3.8
        os.makedirs(self.build_temp, exist_ok=True)

        # build/lib.linux-x86_64-3.8
        os.makedirs(self.build_lib, exist_ok=True)

        kaldifeat_dir = Path(__file__).parent.parent.resolve()

        cmake_args = os.environ.get("KALDIFEAT_CMAKE_ARGS", "")
        make_args = os.environ.get("KALDIFEAT_MAKE_ARGS", "")
        system_make_args = os.environ.get("MAKEFLAGS", "")

        if cmake_args == "":
            cmake_args = "-DCMAKE_BUILD_TYPE=Release"

        extra_cmake_args = " -Dkaldifeat_BUILD_TESTS=OFF "
        extra_cmake_args += f" -DCMAKE_INSTALL_PREFIX={Path(self.build_lib).resolve()}/kaldifeat "  # noqa

        if "PYTHON_EXECUTABLE" not in cmake_args:
            print(f"Setting PYTHON_EXECUTABLE to {sys.executable}")
            cmake_args += f" -DPYTHON_EXECUTABLE={sys.executable}"

        cmake_args += extra_cmake_args

        if is_windows():
            build_cmd = f"""
                cmake {cmake_args} -B {self.build_temp} -S {kaldifeat_dir}
                cmake --build {self.build_temp} --target _kaldifeat --config Release -- -m
                cmake --build {self.build_temp} --target install --config Release -- -m
            """
            print(f"build command is:\n{build_cmd}")
            ret = os.system(
                f"cmake {cmake_args} -B {self.build_temp} -S {kaldifeat_dir}"
            )
            if ret != 0:
                raise Exception("Failed to configure kaldifeat")

            ret = os.system(
                f"cmake --build {self.build_temp} --target _kaldifeat --config Release -- -m"
            )
            if ret != 0:
                raise Exception("Failed to build kaldifeat")

            ret = os.system(
                f"cmake --build {self.build_temp} --target install --config Release -- -m"
            )
            if ret != 0:
                raise Exception("Failed to install kaldifeat")
        else:
            if make_args == "" and system_make_args == "":
                print("For fast compilation, run:")
                print(
                    'export KALDIFEAT_MAKE_ARGS="-j"; python setup.py install'
                )
                make_args = " -j4 "
                print("Setting make_args to '-j4'")

            build_cmd = f"""
                cd {self.build_temp}

                cmake {cmake_args} {kaldifeat_dir}


                make {make_args} _kaldifeat install
            """
            print(f"build command is:\n{build_cmd}")

            ret = os.system(build_cmd)
            if ret != 0:
                raise Exception(
                    "\nBuild kaldifeat failed. Please check the error message.\n"
                    "You can ask for help by creating an issue on GitHub.\n"
                    "\nClick:\n\thttps://github.com/csukuangfj/kaldifeat/issues/new\n"  # noqa
                )



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)

import pickle
from pathlib import Path

import torch
from utils import get_devices, read_ark_txt, read_wave

import kaldifeat

cur_dir = Path(__file__).resolve().parent


def test_online_plp(
    opts: kaldifeat.PlpOptions,
    wave: torch.Tensor,
    cpu_features: torch.Tensor,
):
    """
    Args:
      opts:
        The options to create the online plp extractor.
      wave:
        The input 1-D waveform.
      cpu_features:
        The groud truth features that are computed offline
    """
    online_plp = kaldifeat.OnlinePlp(opts)

    num_processed_frames = 0
    i = 0  # current sample index to feed
    while not online_plp.is_last_frame(num_processed_frames - 1):
        while num_processed_frames < online_plp.num_frames_ready:
            # There are new frames to be processed
            frame = online_plp.get_frame(num_processed_frames)
            assert torch.allclose(
                frame.squeeze(0), cpu_features[num_processed_frames], atol=1e-3
            )
            num_processed_frames += 1

        # Simulate streaming . Send a random number of audio samples
        # to the extractor
        num_samples = torch.randint(300, 1000, (1,)).item()

        samples = wave[i : (i + num_samples)]  # noqa
        i += num_samples
        if len(samples) == 0:
            online_plp.input_finished()
            continue

        online_plp.accept_waveform(16000, samples)

    assert num_processed_frames == online_plp.num_frames_ready
    assert num_processed_frames == cpu_features.size(0)


def test_plp_default():
    print("=====test_plp_default=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-plp.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.PlpOptions()
        opts.frame_opts.dither = 0
        opts.device = device
        plp = kaldifeat.Plp(opts)

        features = plp(wave.to(device))
        if device.type == "cpu":
            cpu_features = features

        assert torch.allclose(features.cpu(), gt, rtol=1e-1)

    opts = kaldifeat.PlpOptions()
    opts.frame_opts.dither = 0

    test_online_plp(opts, wave, cpu_features)


def test_plp_no_snip_edges():
    print("=====test_plp_no_snip_edges=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-plp-no-snip-edges.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.PlpOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.frame_opts.snip_edges = False

        plp = kaldifeat.Plp(opts)

        features = plp(wave.to(device))
        if device.type == "cpu":
            cpu_features = features
        assert torch.allclose(features.cpu(), gt, atol=1e-1)

    opts = kaldifeat.PlpOptions()
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False

    test_online_plp(opts, wave, cpu_features)


def test_plp_htk_10_ceps():
    print("=====test_plp_htk_10_ceps=====")
    filename = cur_dir / "test_data/test.wav"
    wave = read_wave(filename)
    gt = read_ark_txt(cur_dir / "test_data/test-plp-htk-10-ceps.txt")

    cpu_features = None
    for device in get_devices():
        print("device", device)
        opts = kaldifeat.PlpOptions()
        opts.device = device
        opts.htk_compat = True
        opts.num_ceps = 10
        opts.frame_opts.dither = 0

        plp = kaldifeat.Plp(opts)

        features = plp(wave.to(device))
        if device.type == "cpu":
            cpu_features = features
        assert torch.allclose(features.cpu(), gt, atol=1e-1)

    opts = kaldifeat.PlpOptions()
    opts.htk_compat = True
    opts.num_ceps = 10
    opts.frame_opts.dither = 0

    test_online_plp(opts, wave, cpu_features)


def test_pickle():
    for device in get_devices():
        opts = kaldifeat.PlpOptions()
        opts.device = device
        opts.frame_opts.dither = 0
        opts.frame_opts.snip_edges = False

        plp = kaldifeat.Plp(opts)
        data = pickle.dumps(plp)
        plp2 = pickle.loads(data)

        assert str(plp.opts) == str(plp2.opts)

    opts = kaldifeat.PlpOptions()
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False

    plp = kaldifeat.OnlinePlp(opts)
    data = pickle.dumps(plp)
    plp2 = pickle.loads(data)

    assert str(plp.opts) == str(plp2.opts)


if __name__ == "__main__":
    test_plp_default()
    test_plp_no_snip_edges()
    test_plp_htk_10_ceps()
    test_pickle()



# Copyright (c)  2022  Xiaomi Corporation (authors: Fangjun Kuang)

from typing import List

import torch


class OnlineFeature(object):
    """Offline feature is a base class of other feature computers,
    e.g., Fbank, Mfcc.

    This class has two fields:

        (1) opts. It contains the options for the feature computer.
        (2) computer. The actual feature computer. It should be
            instantiated by subclasses.

    Caution:
      It supports only CPU at present.
    """

    def __init__(self, opts):
        assert opts.device.type == "cpu"

        self.opts = opts

        # self.computer is expected to be set by subclasses
        self.computer = None

    @property
    def num_frames_ready(self) -> int:
        """Return the number of ready frames.

        It can be updated by :method:`accept_waveform`.

        Note:
          If you set ``opts.frame_opts.max_feature_vectors``, then
          the valid frame indexes are in the range.
          ``[num_frames_ready - max_feature_vectors, num_frames_ready)``

          If you leave ``opts.frame_opts.max_feature_vectors`` to its default
          value, then the range is ``[0, num_frames_ready)``
        """
        return self.computer.num_frames_ready

    def is_last_frame(self, frame: int) -> bool:
        """Return True if the given frame is the last frame."""
        return self.computer.is_last_frame(frame)

    def get_frame(self, frame: int) -> torch.Tensor:
        """Get the frame by its index.
        Args:
          frame:
            The frame index. If ``opts.frame_opts.max_feature_vectors`` is
            -1, then its valid values are in the range
            ``[0, num_frames_ready)``. Otherwise, the range is
            ``[num_frames_ready - max_feature_vectors, num_frames_ready)``.
        Returns:
          Return a 2-D tensor with shape ``(1, feature_dim)``
        """
        return self.computer.get_frame(frame)

    def get_frames(self, frames: List[int]) -> List[torch.Tensor]:
        """Get frames at the given frame indexes.
        Args:
          frames:
            Frames whose indexes are in this list are returned.
        Returns:
          Return a list of feature frames at the given indexes.
        """
        return self.computer.get_frames(frames)

    def accept_waveform(
        self, sampling_rate: float, waveform: torch.Tensor
    ) -> None:
        """Send audio samples to the extractor.
        Args:
          sampling_rate:
            The sampling rate of the given audio samples. It has to be equal
            to ``opts.frame_opts.samp_freq``.
          waveform:
            A 1-D tensor of shape (num_samples,). Its dtype is torch.float32
            and has to be on CPU.
        """
        self.computer.accept_waveform(sampling_rate, waveform)

    def input_finished(self) -> None:
        """Tell the extractor that no more audio samples will be available.
        After calling this function, you cannot invoke ``accept_waveform``
        again.
        """
        self.computer.input_finished()

    @property
    def dim(self) -> int:
        """Return the feature dimension of this extractor"""
        return self.computer.dim

    @property
    def frame_shift_in_seconds(self) -> int:
        """Return frame shift in seconds of this extractor"""
        return self.computer.frame_shift_in_seconds

    def __getstate__(self):
        return self.opts.as_dict()
